# YASARA MOVIE
# TOPIC:    2. Molecular Graphics
# TITLE:    2.2. Surfaces I - Introduction
# REQUIRES: View 8.11.1
# AUTHOR:   Elmar Krieger
# LICENSE:  GPL
# DESCRIPTION: An introduction to the different kinds of surfaces. Many thanks to Sven Geier for his 'Speed' background (more at www.sgeier.net/fractals).

# Change working directory to the place of this macro
# (so that we don't have to specify paths when loading stuff).
CD (MacroDir)
RequireVersion 4.4.25
OnError Continue
CoordSys Left
Projection perspective
LightSource 45,45,Ambience=0,Shadow=40
Fog 70
AtomSize 100
SurfPar Probe=1.4

# Fresh start
TitleScreen:
Clear
LoadPNG speed
ShowImage speed
UserInput Off
HideMessage

# Don't show console and head up display
HUD Off
MarkAtom None
Console Off
BallStickRadius 50,50
StickRadius 50

# Show the "Surfaces" logo
LoadSce surfaces
# Define start and direction along the plane
start()=-30,-29,-57
dir()=0.239,0.239,0.941
SwitchObj 3-5,off
Style Ball
PosObj surfaces,(start)
OriObj surfaces,Alpha=111.8,Beta=104.4,Gamma=-14.35
PosObj subtitle,0,70,0
OriObj subtitle,40,-80,50
AutoPosObj surfaces,(start+dir*196),Steps=150
AutoPosObj surfaces,20,25,80,Steps=100,Wait=No
AutoPosObj subtitle,37,13,88,Steps=100,Wait=No
AutoOriObj surfaces subtitle,11,322,16,Steps=100
Wait 50
# Show 1crn
PosObj 1crn,0,-6,-8
OriObj 1crn,11,322,16
SwitchObj 1crn,on
StickObj 1crn

AutoMove Z=0.1
Wait 300
AutoMove Z=0

# Show solvent accessible surface
SpeedUp Off
TransferObj MolSrf03,1crn
AutoRotateObj 1crn MolSrf03,Y=0.15
SurfPar resolution=3
ColorSurf accessible,Green,100,Blue,100,Specular=Yes
first,last=SpanAtom Obj 1crn
for i=first to last-1 step 2
  ShowSurfAtom (i) (i+1),accessible
  Wait 1
Wait 30

# Look into the protein
AutoPos Z=63,Steps=100
Wait 100
AutoPos Z=68,Steps=100
Wait 60
for i=100 to 50 step -1
  ColorSurfObj 1crn,accessible,Green,(i),Blue,100
  Wait 1
Wait 25
for i=50 to 100
  ColorSurfObj 1crn,accessible,Green,(i),Blue,100
  Wait 1
Wait 25
SwitchObj MolSrf03,On
for i=100 to 0 step -1
  ColorSurfObj 1crn,accessible,Green,(i),Blue,(i)
  Wait 1
Wait 25
for i=100 to 80 step -1
  ColorSurfObj MolSrf03,molecular,OutAlpha=(i),InAlpha=100
  Wait 1
AutoPosObj 3-4,Y=-8.5,Steps=30
# Take snapshot here, Eliza!
SpeedUp On
ShowMessage 'Welcome to a 10 minute tutorial about surfaces, (FirstName)!' 
  
# Animate the logo
ShowButton Continue
while 1
  for style in 'BallStick','Stick','BallStick','Ball'
    for i=01 to 12
      (style)Atom C(i)
      Wait 2
      if ContinueButton
        break 3
AutoMove (-dir)
Wait 150

# Explain surface types
SurfaceTypes:
DelAll
SpeedUp On
UserInput On
Console Off
LightSource 45,45,Ambience=85,Shadow=65
LoadSce surfaces
Style Ribbon,Stick
ColorSurfObj 4,OutAlpha=85
DelObj 1-2
SwitchObj 4,off
HideObj 5
Pos X=311,Y=300,Z=1140
AutoPos X=0,Y=0,Z=38,Steps=150
for i=1 to 46
  HideSecRes (i) Obj 1crn
  ShowRes (i) Obj 1crn
  BallRes (i) Obj 1crn
  Wait 1
Style Ball
HideObj 5

textcom="You are looking at Crambin in space-filling mode. YASARA displays atoms with 66% of their real Van der Waals size.",'SurfPar Probe=1.4,Resolution=3,Molecular=Gaussian',
        "This makes sure that the view stays informative: overlapping atoms are either bound or make a bad close contact (bump/clash).",'ColorSurf VdW,Red,75,White,100,1',
        "Let's compare that to the real atom sizes outlined by a Van der Waals surface.",'pass',
        "Click View > Show surface > Object, double-click 1crn, check only the 'Van der Waals surface' box, click OK.",'ShowSurfObj 1crn,VdW',
        "The Van der Waals surface outlines the volume occupied by the protein atoms. Significantly more than indicated by the spheres.",'StickAll',
        "Let's color the surface: Click View > Color > All surfaces > OK > AtomCol > Continue with inside > double-click Blue.",'ColorSurfObj 1crn,VdW,AtomCol,100,Blue,100',
        "Explore the inside of the protein. You can see that there are lots of empty spaces between the atoms.",'ShowRes HOH',
        "Proteins live in solution, they are in permanent contact with water molecules that roll over the Van der Waals surface.",'ShowSurfAtom O Res HOH,VdW,OutCol=Red,OutAlpha=60,InCol=White,InAlpha=40,1',
        "Water and protein atoms cannot get much closer than the sum of their VdW radii, except when they form hydrogen bonds.",'HideSurfObj HOHShell,All',
        "Let's look at the movements of some of the water molecules...",'pass'
for i=1 to 19 step 2
  ShowMessage (textcom(i))
  Wait ContinueButton
  (textcom(i+1))

AutoPos 0,0,38,Steps=50
# Pick 20 waters that roll over the surface
waters=20
for i=1 to waters
  waterlist(i)=325+(174//waters)*i*3
waters=count waterlist
HideRes HOH
ColorSurfObj 4,Molecular,AtomCol,50,Blue,50
for water in waterlist
  ShowRes Atom (water)
steps=30
textcom="As you can see, the waters cannot enter the empty space inside the protein.",'ShowSurfAtom O Res HOH visible,VdW,OutCol=Red,OutAlpha=60,InCol=White,InAlpha=40,1',
        "Their Van der Waals spheres are simply too large to fit into the gaps.",'SwitchObj 4,on',
        "For the waters, the protein appears as a closed body that excludes the solvent.",'ColorSurfObj 4,Molecular,AtomCol,100,Blue,100',
        "The resulting surface is called 'solvent excluded surface', 'Connolly surface' or 'molecular surface'.",'HideSurfObj 3',
        "Calculating this surface is computationally expensive, that's why YASARA also supports a faster approximation:",'ShowSurfObj 3,molecular,dynamic,AtomCol,100,Blue,100',
        "The 'Gaussian' molecular surface can be activated by clicking on View > Show surface > Parameters > Gaussian.",'SwitchObj 4,Off',
        "Both look rather similar, and calculated surface areas usually differ by only 2%.",'pass',
        "In practice, best choose the 'numeric molecular surface', which is fast and exact.",'pass'
for t=1 to 13 step 2
  ShowMessage (textcom(t))
  ShowButton Continue
  while 1
    # Pick new water positions and calculate rotation axes
    for i=1 to waters
      # Choose a random close water
      startwater=waterlist(i)
      closewaterlist()=ListAtom O Res HOH with distance < 6 from (startwater) and not (startwater)
      if count closewaterlist
        endwater=onein closewaterlist
        startpos()=PosAtom (startwater)
        endpos()=PosAtom (endwater)
        for k=1 to 3
          step(i)(k)=(endpos(k)-startpos(k))/steps
        # Calculate the water rotation axis
        # This is just the vector product of the direction vector and the position
        axis(i)1=step(i)2*startpos3-step(i)3*startpos2
        axis(i)2=step(i)3*startpos1-step(i)1*startpos3
        axis(i)3=step(i)1*startpos2-step(i)2*startpos1
      else
        endwater=startwater
        step(i)()=0,0,0
        axis(i)()=1,0,0
    # Move the waters    
    for i=1 to steps
      for j=1 to waters
        water=waterlist(j)
        MoveRes Atom (water),(step(j)1),(step(j)2),(step(j)3)
        pos()=PosAtom (water)
        RotAxisRes Atom (water),(pos1),(pos2),(pos3),(axis(j)1),(axis(j)2),(axis(j)3),-10
      Wait 1   
      if ContinueButton
        break 2
  (textcom(t+1))
HideMessage

# Explain solvent accessible surface
ShowSurfObj 3,VdW,OutCol=AtomCol,100,Blue,100
for i=100 to 0 step -2
  ColorSurfObj 3,molecular,OutAlpha=(i)
  if i<60
    ColorSurfObj 5,VdW,OutAlpha=(i),InAlpha=40
  if i<40
    ColorSurfObj 5,VdW,InAlpha=(i)
  Wait 1
HideSurfObj 3,molecular
SwitchObj 5,off
textcom="In addition to Van der Waals and molecular surface, there is a third popular type: the solvent accessible surface.",'pass',
        "Click View > Show surface > Object, double-click 1crn, check only the 'solvent accessible surface' box, click OK.",'ShowSurfObj 1crn,accessible,OutCol=green,40,white,60',
        "The solvent accessible surface consists of the points closest to the protein that the nuclei of the water oxygens can reach.",'ColorSurfObj 1crn,accessible,OutAlpha=100,InAlpha=100',
        "Calculating this surface is straightforward: just add the radius of a water molecule to the VdW radii of the protein atoms...",'ColorSurfObj 3,accessible,OutCol=AtomCol,InCol=Blue,Specular=No',
        "..and then draw a simple VdW surface. If the water probe radius is 0, the accessible surface is equal to the VdW surface.",'HideSurfObj 3,VdW'
for i=1 to 9 step 2
  ShowMessage (textcom(i))
  Wait ContinueButton
  (textcom(i+1))

ShowButton Continue
while 1
  for range in '1.4 to 0.5 step -0.1','0.5 to 1.4 step 0.1'
    for radius=(range)
      ShowMessage 'You can see the transformation between solvent accessible and VdW surface. The water probe radius is now (radius) A.'
      if ContinueButton
        break 3
      SurfPar Probe=(radius)
      Wait 1
ShowObj 5  
textcom="Now set the water probe radius back to the default 1.4 Angstrom: Click View > Show surface > Parameters.",'SurfPar Probe=1.4',
        "The accessible surface is important, because its area is proportional to the number of water molecules that touch the protein.",'SwitchObj 5,On',
        "These waters have reduced degrees of freedom and thus lower entropy, which is considered by many energy functions.",'ColorRes Obj 3,blue,cyan'
for i=1 to 5 step 2
  ShowMessage (textcom(i))
  Wait ContinueButton
  (textcom(i+1))

if Model
  ShowMessage "That's it for this time. If you want to know more, continue with the next tutorial: 'Surfaces II - Surface environments'"
else
  ShowMessage "That's it for this time. If you want to know more, download the next tutorial 'Surfaces II' from www.yasara.org/movies"
AutoRotate Y=1
first,last=SpanRes HOH Obj HOHShell
for i=first to last
  HideRes (i) Obj HOHShell
  Wait 1
AutoMove (dir*0.1)
for i=100 to 0 step -1
  ColorSurfObj 3,accessible,OutCol=AtomCol,OutAlpha=(i),InCol=Blue,InAlpha=100,Specular=No
  Wait 1
for i=100 to 1 step -1
  ColorSurfObj 3,accessible,OutCol=AtomCol,OutAlpha=0,InCol=Blue,InAlpha=(i)
  if i<47
    HideRes (i) Obj 3
  Wait 1
